#!/usr/bin/env perl
#
# NAME:
#	getroots.pl - fetch the DNS root server list
#
# SYNOPSIS:
#	getroots.pl [options]
#
# DESCRIPTION:
#	This script outputs content suitable for seeding the root zone
#	cache.  It first looks up all "N" (13) root name servers, then
#	asks for the NS list.  This should ensure we get the addresses
#	for each in the response.
#
#	Options:
#
#	-N "N"
#		We'll fetch the first "N" addresses (13).
#
#	-T "TTL"
#		Override the TTL with our "TTL" (8d).
#
# BUGS:
#	We rely on dig(1) for this.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: getroots.pl,v 1.1 2008/01/25 17:56:31 sjg Exp $
#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#


if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}
$Myname =~ s/\.pl//;

push(@INC, ('.', $Mydir, '/usr/local/lib/perl'));

require "$Mydir/getopts.pl";

$opt_N = 13;			# how many root servers
$opt_T = '8d';			# TTL we want
&Getopts('N:T:');

@Alphabet = ('A', 'B', 'C', 'D', 'E', 'F', 'G',
	     'H', 'I', 'J', 'K', 'L', 'M', 'N',
	     'O', 'P', 'Q', 'R', 'S', 'T', 'U',
	     'V', 'W', 'X', 'Y', 'Z');

for ($i = 0; $i < $opt_N; $i++) {
  $name = $Alphabet[$i] . '.ROOT-SERVERS.NET.';
  system("dig $name > /dev/null");
}
if ($opt_T eq '' || $opt_T eq '-') {
  system("dig . ns");
} else {
  system("dig . ns | sed 's,\t[1-9][^ ]*s ,\t$opt_T ,'");
}

