#!/usr/bin/env perl
#
# NAME:
#	slavezones.pl - generate slave.zones
#
# SYNOPSIS:
#	slavezones.pl ["dnsaddr" ...]
#
# DESCRIPTION:
#	This script reads bind-8 master zone definitions from stdin and
#	outputs slave zone ones.
#
#	If "dnsaddr" is omitted then the IP address of the
#	current host is used.
#
#	Basically, each 'master' zone is replaced with a
#	'slave' zone with "dnsaddr" being the address to do zone
#	transfers from.
#
# BUGS:
#	The only advantage of the perl version over the shell script
#	is handling of 'db.auth'.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: slavezones.pl,v 1.4 2001/08/30 07:48:37 sjg Exp $
#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#

if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}

push(@INC, ('.', $Mydir, '/usr/local/lib/perl'));
require 'dofile.pl';

&do_file(".dnsrc.pl",'.',$ENV{'HOME'},'/etc',$Mydir);
&do_file(".${Myname}rc.pl",'.',$ENV{'HOME'},'/etc',$Mydir);

@masters = @ARGV;
if (scalar(@masters) == 0) {
  if ($dnsaddr eq '' && (-s '/etc/.hostsrc')) {
    chop($dnsaddr = `. /etc/.hostsrc; echo \$ADDR`);
  }
  if ($dnsaddr eq '') {
    chop($dnsaddr = `myname=\`uname -n\`; egrep "^[1-9].*[ 	]\$myname([ 	.]|\$)" /etc/hosts | awk '{ print \$1; exit }'`);
  }
  push(@masters, $dnsaddr);
}

print "// generated by $0\n// DO NOT EDIT!!!!\n";

$masters = join(';', @masters);

while (<STDIN>) {
  if (m,^\s*//,) {
    print;
    next;
  }
  if (m/zone\s+"(\S+)"/) {
    $zone = $1;
    print;
    next;
  }
  if (m/type\s+master/) {
    print "\ttype slave; masters { $masters; };\n";
    next;
  }
  if (m/file\s+"(\S+)"/) {
    $file = $1;
    if ($file eq 'db.auth') {
      $file = "secondary/$zone";
    } else {
      $file =~ s,^.*/,secondary/,;
    }
    $file =~ s,\.soa$,,;
    print "\tfile \"$file\";\n";
    next;
  }
  print;
}
